package com.dmartinez_dev.multiplicandos;

import android.content.ClipData;
import android.content.ClipDescription;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.Color;
import android.graphics.drawable.AnimationDrawable;
import android.media.AudioManager;
import android.media.SoundPool;
import android.os.Vibrator;
import android.support.v7.app.ActionBarActivity;
import android.os.Bundle;
import android.support.v7.app.AlertDialog;
import android.util.Log;
import android.view.DragEvent;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.animation.AccelerateInterpolator;
import android.view.animation.AlphaAnimation;
import android.view.animation.Animation;
import android.view.animation.AnimationUtils;
import android.widget.TextView;
import android.widget.Toast;

import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;

import java.lang.reflect.Type;
import java.util.Arrays;
import java.util.Random;
import java.util.Vector;


public class OperacionActivity extends ActionBarActivity implements View.OnDragListener, View.OnLongClickListener {

    Vibrator vibrador;

    private Vector<Integer> _multiplicandos;
    private int _cantidad_operaciones;
    private int _posicion_actual;
    private int _ultimo_indice_multiplicando;

    private TextView textView_numero_operaciones;
    private TextView tvResultado;
    private TextView tvOpcion1;
    private TextView tvOpcion2;
    private TextView tvOpcion3;
    private TextView tvOpcion4;
    private int _respuesta;
    private Resultados _ultimo_calculo;
    private SoundPool _soundpool;
    private int _id_aplausos;
    private int _id_ok;
    private int _id_error;
    private int _ultimo_multiplicador;

    private void findViews()
    {
        textView_numero_operaciones = (TextView) findViewById(R.id.textView_numero_operaciones);
        tvResultado = (TextView)findViewById( R.id.tv_op );
        tvOpcion1 = (TextView)findViewById( R.id.tv_opcion1 );
        tvOpcion2 = (TextView)findViewById( R.id.tv_opcion2 );
        tvOpcion3 = (TextView)findViewById( R.id.tv_opcion3 );
        _posicion_actual = 1;
        Generar_Siguiente_Calculo();
    }

    private void Generar_Siguiente_Calculo()
    {
        int multiplicando = 0;

        //Generamos el siguiente calculo tomando uno de los multiplicandos que se configuro
        if(_multiplicandos.size() == 1)
            multiplicando = _multiplicandos.get(0);
        else //En el caso que tenemos mas de un posible multiplicando
        {
            int indice = generar_Aleatorio_sencillo(_multiplicandos.size(),_ultimo_indice_multiplicando);
            _ultimo_indice_multiplicando = indice;
            multiplicando = _multiplicandos.get(indice-1);

        }

        Resultados r = generar_Aleatorio(multiplicando, _ultimo_multiplicador);
        _ultimo_calculo = r;
        _ultimo_multiplicador = r.getMultiplicador();

        int[] valores = r.get_Menor_Mayor();
        tvOpcion1.setText(String.valueOf(valores[0]));
        tvOpcion2.setText(String.valueOf(valores[1]));
        tvOpcion3.setText(String.valueOf(valores[2]));
        tvResultado.setText(String.format("%s * %s = ?", r.getMultiplicador(), r.getMultiplicando()));
        textView_numero_operaciones.setText(String.format("Operacion: %s / %s", _posicion_actual, _cantidad_operaciones));
        _respuesta = r.getResultado();

    }


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_operacion);

        //Cargamos los sonidos que se van a usar
        _soundpool = new SoundPool(5, AudioManager.STREAM_MUSIC,0);
        _id_aplausos = _soundpool.load(getApplicationContext(),R.raw.applause,0);
        _id_ok = _soundpool.load(getApplicationContext(),R.raw.ok,0);
        _id_error = _soundpool.load(getApplicationContext(),R.raw.error,0);



        //Se leen los extras que enviamos desde la Activity Anterior
        Intent i = getIntent();
        _cantidad_operaciones = i.getIntExtra(DificultadActivity.EXTRA_CANTIDAD_OPERACIONES,0);
        String Json_multiplicandos = i.getStringExtra(DificultadActivity.EXTRA_MULTIPLICANDOS);

        Type collectionType = new TypeToken<Vector<Integer>>(){}.getType();
        Gson gson = new Gson();
        _multiplicandos = gson.fromJson(Json_multiplicandos,collectionType);
        _posicion_actual = 0;

        findViews();


        tvOpcion1.setOnLongClickListener(this);
        tvOpcion2.setOnLongClickListener(this);
        tvOpcion3.setOnLongClickListener(this);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_operacion, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_salir) {
            finish();
            return true;
        }

        return super.onOptionsItemSelected(item);
    }


    @Override
    public boolean onDrag(View v, DragEvent event)
    {
        // Defines a variable to store the action type for the incoming event
        final int action = event.getAction();

        // Handles each of the expected events
        switch(action) {

            case DragEvent.ACTION_DRAG_STARTED:

                // Determines if this View can accept the dragged data
                if (event.getClipDescription().hasMimeType(ClipDescription.MIMETYPE_TEXT_PLAIN)) {

                    v.setBackgroundResource(R.drawable.animation);
                    ((AnimationDrawable) v.getBackground()).start();

                    // Invalidate the view to force a redraw in the new tint
                    v.invalidate();

                    // returns true to indicate that the View can accept the dragged data.
                    return true;

                }

                // Returns false. During the current drag and drop operation, this View will
                // not receive events again until ACTION_DRAG_ENDED is sent.
                return false;

            case DragEvent.ACTION_DRAG_ENTERED:

                // Ignore the event
                return true;

            case DragEvent.ACTION_DRAG_LOCATION:

                // Ignore the event
                return true;

            case DragEvent.ACTION_DRAG_EXITED:

                // Ignore the event
                return true;

            case DragEvent.ACTION_DROP:

                // Gets the item containing the dragged data
                ClipData.Item item = event.getClipData().getItemAt(0);

                // Gets the text data from the item.
                String dragData = item.getText().toString();

                tvResultado.setText(String.format("%s * %s = %s", _ultimo_calculo.getMultiplicador(), _ultimo_calculo.getMultiplicando(), dragData));

                if(Integer.parseInt(dragData) == _respuesta) {
                    v.setBackgroundColor(Color.GREEN);
                    _soundpool.play(_id_ok,1,1,1,0,1);

                    //Asignamos animacion
                    Animation fadeOut = new AlphaAnimation(1, 0);
                    fadeOut.setInterpolator(new AccelerateInterpolator()); //and this
                    fadeOut.setDuration(1200);
                    fadeOut.setAnimationListener(new Animation.AnimationListener() {
                        @Override
                        public void onAnimationStart(Animation animation) {

                        }

                        @Override
                        public void onAnimationEnd(Animation animation) {
                            if (_posicion_actual < _cantidad_operaciones) {
                                _posicion_actual++;
                                Generar_Siguiente_Calculo();
                                tvResultado.setBackgroundDrawable(null);

                            }
                            else if (_posicion_actual == _cantidad_operaciones)
                            {
                                _soundpool.play(_id_aplausos,1,1,1,0,1);
                                Crear_Dialogo_Felicitacion();
                            }
                        }

                        @Override
                        public void onAnimationRepeat(Animation animation) {

                        }
                    });

                    v.startAnimation(fadeOut);


                }

                else {
                    v.setBackgroundColor(Color.RED);
                    _soundpool.play(_id_error,1,1,1,0,1);
                    Animation animacion_shake = AnimationUtils.loadAnimation(this,R.anim.shake);
                    animacion_shake.setAnimationListener(new Animation.AnimationListener() {
                        @Override
                        public void onAnimationStart(Animation animation) {

                        }

                        @Override
                        public void onAnimationEnd(Animation animation) {
                            tvResultado.setBackgroundDrawable(null);
                            tvResultado.setText(String.format("%s * %s = ?", _ultimo_calculo.getMultiplicador(), _ultimo_calculo.getMultiplicando()));
                        }

                        @Override
                        public void onAnimationRepeat(Animation animation) {

                        }
                    });
                    v.startAnimation(animacion_shake);

                    try {
                        if (vibrador == null) {
                            vibrador = (Vibrator) OperacionActivity.this.getApplicationContext().getSystemService(Context.VIBRATOR_SERVICE);
                        }
                        if (vibrador != null) {
                            if (vibrador.hasVibrator()) {
                                vibrador.vibrate(500);
                            }
                        }
                    }
                    catch (Exception error)
                    {
                        Toast.makeText(getApplicationContext(),"No se pudo utilizar el vibrador",Toast.LENGTH_LONG).show();
                        return true;
                    }

                }


                // Invalidates the view to force a redraw
                v.invalidate();

                // Returns true. DragEvent.getResult() will return true.
                return true;

            case DragEvent.ACTION_DRAG_ENDED:

                // Invalidates the view to force a redraw
                v.invalidate();

                // Does a getResult(), and displays what happened.
                if (event.getResult())
                {
                    //Aqui el arrastre fue controlado
                }
                else
                {
                    //Aqui el arrastre no fue controlado
                    tvResultado.setBackgroundDrawable(null);
                }

                //Le quitamos los colores de fondo a las distintas opciones
                tvOpcion1.setBackgroundDrawable(null);
                tvOpcion2.setBackgroundDrawable(null);
                tvOpcion3.setBackgroundDrawable(null);

                // returns true; the value is ignored.
                return true;

            // An unknown action type was received.
            default:
                Log.e("DragDrop Example", "Unknown action type received by OnDragListener.");
                break;
        }

        return false;
    }

    public void Crear_Dialogo_Felicitacion() {
        // Use the Builder class for convenient dialog construction
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle("Completado");
        builder.setMessage("En hora buena has completado los Ejercicios.!!!")
                .setPositiveButton("Salir", new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int id) {
                        finish();
                    }
                });


        // Create the AlertDialog object and return it
        AlertDialog dialogo =  builder.create();
        dialogo.show();
    }


    private Resultados generar_Aleatorio(int multiplicando,int multiplicador_anterior)
    {
        Resultados r = new Resultados();
        Random rn = new Random();

        int calculo_temporal = 0;
        int maximum = 9;
        int minimum = 1;
        int randomNum = 0;



        int range = maximum - minimum + 1;

        r.setMultiplicando(multiplicando);
        //Calculamos el multiplicador
        while(true)
        {
            randomNum = rn.nextInt(range) + minimum;
            if(randomNum != multiplicador_anterior)
            {
                break;
            }
        }
        r.setMultiplicador(randomNum);

        //Calculamos el resultado
        r.setResultado(r.getMultiplicando() * r.getMultiplicador());

        //Calculamos el primer resultado falso
        while(true)
        {
            randomNum = rn.nextInt(range) + minimum;
            calculo_temporal = (randomNum * r.multiplicando);
            if(calculo_temporal != r.resultado) //Si el calculo del numero aleatorio por el multiplicando es distinto al resultado real
            {
                break;
            }
        }
        r.setFalso1(calculo_temporal);

        //Calculamos el segundo resultado falso
        while(true)
        {
            randomNum = rn.nextInt(range) + minimum;
            calculo_temporal = (randomNum * r.multiplicando);
            if((calculo_temporal != r.resultado) && (calculo_temporal != r.getFalso1())) //Si el calculo del numero aleatorio por el multiplicando es distinto al resultado real
            {
                break;
            }
        }
        r.setFalso2(calculo_temporal);

        return r;


    }


    private int generar_Aleatorio_sencillo(int maximo,int exclusion)
    {
        Random rn = new Random();

        int maximum = maximo;
        int minimum = 1;
        int randomNum = 0;

        int range = maximum - minimum + 1;

        //Calculamos el multiplicador
        while(true)
        {
            randomNum = rn.nextInt(range) + minimum;
            if(randomNum != exclusion)
            {
                break;
            }
        }

        return randomNum;

    }


    @Override
    public boolean onLongClick(View v) {
        TextView control = (TextView)v;

        ClipData.Item item = new ClipData.Item((String) control.getText().toString());
        ClipData clipData = new ClipData((CharSequence) control.getText().toString(),
                new String[]{ClipDescription.MIMETYPE_TEXT_PLAIN}, item);

        control.setBackgroundColor(Color.argb(100, 255, 100, 50));

        tvResultado.setOnDragListener(OperacionActivity.this);
        control.startDrag(clipData, new View.DragShadowBuilder(control), null, 0);

        return true;
    }

    class Resultados
    {
        private int multiplicando;
        private int multiplicador;
        private int resultado;
        private int falso1;
        private int falso2;

        public int[] get_Menor_Mayor()
        {
            int iArr[] = {this.resultado, this.falso1, this.falso2};
            Arrays.sort(iArr);
            return iArr;
        }

        public int getMultiplicando() {
            return multiplicando;
        }

        public void setMultiplicando(int multiplicando) {
            this.multiplicando = multiplicando;
        }

        public void setMultiplicador(int multiplicador) {
            this.multiplicador = multiplicador;
        }

        public void setResultado(int resultado) {
            this.resultado = resultado;
        }

        public void setFalso1(int falso1) {
            this.falso1 = falso1;
        }

        public void setFalso2(int falso2) {
            this.falso2 = falso2;
        }


        public int getMultiplicador() {
            return multiplicador;
        }

        public int getResultado() {
            return resultado;
        }

        public int getFalso1() {
            return falso1;
        }

        public int getFalso2() {
            return falso2;
        }


    }
}
